//***********************************************************************************//
//*	Project:																		*//
//*																					*//
//*	File:		Console.c			 												*//
//*	Version:	1.0																	*//
//*																					*//
//*	Author:		Bob Marlowe															*//
//*	(C) 2015	Jörg Meier Software - Entwicklung briefe@jmeiersoftware.de			*//
//***********************************************************************************//
//***********************************************************************************//
//*																					*//
//*		Principles of Operation														*//
//*																					*//
//*		Enable printf() and scanf() to be used by Creator programs					*//
//*																					*//
//*		At initialization pointers to the needed functions for 						*//
//*		checking for a byte ready, read a byte and write a byte are provided		*//
//*		scanf() and printf() will call the provided functions _read() and _write()	*//
//*		which are provided here. It is required to #includr <stdio.h>.				*//
//*		Newlib nano and when floats are involved newlib nano float formatting		*//
//*		should be enabled 																			*//
//*																					*//
//***********************************************************************************//

//***********************************************************************************//
//*************************** Libraries *********************************************//
//***********************************************************************************//
#include <project.h>
#include <Console.h>

//***********************************************************************************//
//********************* Definitions and Macros **************************************//
//***********************************************************************************//
#define Wait(x)										//	Used for better readability

//***********************************************************************************//
//******************* Structs, Typedefs and Enums ***********************************//
//***********************************************************************************//
//	None yet

//***********************************************************************************//
//********************** External Data Area *****************************************//
//***********************************************************************************//
//	None yet

//***********************************************************************************//
//*************************** Data Area *********************************************//
//***********************************************************************************//
static GetByte		FuncGetByte		=	NULL;		//	Function pointer to read a byte
static ByteReady	FuncByteReady	=	NULL;		//	...to check for byte ready
static PutByte		FuncPutByte		=	NULL;		//	...and to write a byte


//***********************************************************************************//
//************************ Program Area *********************************************//
//***********************************************************************************//

//***********************************************************************************//
void Console_Start(ByteReady Ready, GetByte Get, PutByte Put)
//*	Populate the required function pointers 										*//
//***********************************************************************************//
{
	FuncByteReady	= Ready;
	FuncGetByte		= Get;
	FuncPutByte		= Put;
}
//***********************************************************************************//


//***********************************************************************************//
int _read (int fd, const void *buf, size_t count)									 //
//*	Read count bytes into buffer													*//
//*	File descriptor fd is not used													*//
//***********************************************************************************//
{
size_t CharCnt = 0;
	if((FuncByteReady == NULL) | (FuncGetByte == NULL)) return 0;
	
	(void)fd;								//	Parameter is not used, suppress unused argument warning
	for (;count > 0; --count) 
	{
		while(!FuncByteReady()) Wait();
		*((uint8*)buf++) = FuncGetByte();	//	Save character received by UARTx device into the receive buffer
		CharCnt++;							//	Increase char counter
	}
	return CharCnt;							//	Return number of characters red
}
//***********************************************************************************//

//***********************************************************************************//
int _write (int fd, const void *buf, size_t count)
//*	Write count bytes from buffer													*//
//*	File descriptor fd is not used													*//
//***********************************************************************************//
{
	fd = count;								//	Parameter is not used, take for own purposes
	
	while(count-- > 0) 
	{
		FuncPutByte(*(char*)buf++);			//	Write it out
		fd++;								//	Count transmitted characters
	}
	return fd;								//	Return number of characters written
}
//***********************************************************************************//

